<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 *  @package   jchoptimize/core
 *  @author    Samuel Marshall <samuel@jch-optimize.net>
 *  @copyright Copyright (c) 2024 Samuel Marshall / JCH Optimize
 *  @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Core\Css\Components;

use CodeAlfa\RegexTokenizer\Css;
use JchOptimize\Core\Css\CssComponents;
use JchOptimize\Core\Exception\InvalidArgumentException;

class CssRule implements CssComponents
{
    use Css;

    protected string $selectorList;
    protected string $declarationList;
    public function __construct($selectorList = '', $declarationList = '')
    {
        $this->selectorList = $selectorList;
        $this->declarationList = $declarationList;
    }
    public static function load(string $css): static
    {
        $cssRuleRegex = self::cssRuleWithCaptureValueToken();
        if (!\preg_match("#^{$cssRuleRegex}\$#s", $css, $matches)) {
            throw new InvalidArgumentException('Invalid CSS rule: ' . $css);
        }
        $selectorList = $matches['selectorlist'];
        $declarationList = $matches['declarationlist'];
        return new static($selectorList, $declarationList);
    }
    public function render(): string
    {
        return "{$this->selectorList}{{$this->declarationList}}";
    }
    private static function cssRuleWithCaptureValueToken(): string
    {
        $selectors = self::cssSelectorListToken();
        return "(?<selectorlist>{$selectors}){(?<declarationlist>.*)}";
    }
    public function getSelectorList(): string
    {
        return $this->selectorList;
    }
    public function setSelectorList(string $selectorList): void
    {
        $this->selectorList = $selectorList;
    }
    public function getDeclarationList(): string
    {
        return $this->declarationList;
    }
    public function setDeclarationList(string $declarationList): void
    {
        $this->declarationList = $declarationList;
    }
}
